from django.db.models import Count

from netbox.views import generic
from . import filtersets, forms, models, tables
from .api.serializers import AWXSerializer, AWXInventorySerializer
from django.shortcuts import redirect, render
from core.models import Job
from .models import AWXInventory
from .synchronization import sync_all
import logging
from django.contrib import messages

logger = logging.getLogger(__name__)


class AWXView(generic.ObjectView):
    queryset = models.AWX.objects.all()
    serializer_class = AWXSerializer


class AWXListView(generic.ObjectListView):
    queryset = models.AWX.objects.all()
    table = tables.AWXTable


class AWXEditView(generic.ObjectEditView):
    queryset = models.AWX.objects.all()
    form = forms.AWXForm


class AWXDeleteView(generic.ObjectDeleteView):
    queryset = models.AWX.objects.all()


class AWXInventoryView(generic.ObjectView):
    queryset = models.AWXInventory.objects.all()
    serializer_class = AWXInventorySerializer


class AWXInventoryListView(generic.ObjectListView):
    queryset = models.AWXInventory.objects.all()
    table = tables.AWXInventoryTable


class AWXInventoryEditView(generic.ObjectEditView):
    queryset = models.AWXInventory.objects.all()
    form = forms.AWXInventoryForm


class AWXInventoryDeleteView(generic.ObjectDeleteView):
    queryset = models.AWXInventory.objects.all()


class AWXInventorySyncView(generic.ObjectView):
    queryset = models.AWXInventory.objects.all()
    serializer_class = AWXInventorySerializer

    def post(self, request, pk, *args, **kwargs):
        logger.info("Sync inventory")
        inventory = AWXInventory.objects.get(id=pk)
        job = Job.enqueue(sync_all, instance=inventory, name="Full sync of inventory")
        messages.success(request, f"Queued job #{job.pk} to sync inventory {inventory}")

        return render(
            request,
            self.get_template_name(),
            {
                "object": inventory,
                "tab": self.tab,
                **self.get_extra_context(request, inventory),
            },
        )
